package cn.iocoder.yudao.module.depository.dal.mysql.warehouse;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaQueryVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 库区库域 Mapper
 * @author wlh
 */
@Mapper
public interface WarehouseAreaMapper extends AbstractMapper<WarehouseAreaDO> {
    @Override
    default PageResult<WarehouseAreaDO> selectPage(PageVO page, Object object) {
        if (object instanceof WarehouseAreaQueryVO) {
            WarehouseAreaQueryVO vo = (WarehouseAreaQueryVO)object;
            return selectPage(page, new LambdaQuery<WarehouseAreaDO>()
                .eqIfPresent(WarehouseAreaDO::getWarehouseId, vo.getWarehouseId())
                .likeIfPresent(WarehouseAreaDO::getName, vo.getName())
                .eqIfPresent(WarehouseAreaDO::getCode, vo.getCode())
                .eqIfPresent(WarehouseAreaDO::getPid, vo.getPid())
                .eqIfPresent(WarehouseAreaDO::getStatus, vo.getStatus())
                    .eq(WarehouseAreaDO::getPid, 0)
                .betweenIfPresent(WarehouseAreaDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(WarehouseAreaDO::getId));
        }

        return null;
    }

    @Override
    default List<WarehouseAreaDO> selectList(Object object) {
        if (object instanceof WarehouseAreaQueryVO) {
            WarehouseAreaQueryVO vo = (WarehouseAreaQueryVO)object;
            return selectList(new LambdaQuery<WarehouseAreaDO>()
                .eqIfPresent(WarehouseAreaDO::getWarehouseId, vo.getWarehouseId())
                .likeIfPresent(WarehouseAreaDO::getName, vo.getName())
                .eqIfPresent(WarehouseAreaDO::getCode, vo.getCode())
                .eq(WarehouseAreaDO::getPid, 0)
                .eqIfPresent(WarehouseAreaDO::getStatus, vo.getStatus())
                .betweenIfPresent(WarehouseAreaDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(WarehouseAreaDO::getId));
        }

        return null;
    }

    @ResultType(WarehouseAreaDO.class)
    @Select({
            "<script>",
            "select * ",
            "FROM ecw_warehouse_area ewa ",
            "WHERE ewa.pid=0",
            "and ewa.deleted=0 ",
            "<when test = 'query.warehouseId != null'>",
            "AND ewa.`warehouse_id` = #{query.warehouseId}",
            "</when>",
            "<when test = 'query.cityId != null'>",
            "AND exists (SELECT id from ecw_warehouse where shi=#{query.cityId} and ewa.warehouse_id=id)",
            "</when>",
            "<when test = 'query.name != null and query.name != \"\" '>",
            "AND ewa.`name` like concat('%',concat(#{query.name},'%'))",
            "</when>",
            "<when test = 'query.code != null and query.code != \"\" '>",
            "AND ewa.`code` like concat('%',concat(#{query.code},'%'))",
            "</when>",
            "order by ewa.id desc",
            "</script>"
    })
    List<WarehouseAreaDO> selectListV1(@Param("query") WarehouseAreaQueryVO query);
}
