package cn.iocoder.yudao.module.depository.service.lineChannelPackaging;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPriceStepClearance.LineChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.depository.dal.mysql.lineChannelPriceStepClearance.LineChannelPriceStepClearanceMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.depository.vo.lineChannelPackaging.*;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPackaging.LineChannelPackagingDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.depository.convert.lineChannelPackaging.LineChannelPackagingConvert;
import cn.iocoder.yudao.module.depository.dal.mysql.lineChannelPackaging.LineChannelPackagingMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.depository.enums.ErrorCodeConstants.*;

/**
 * 线路渠道清关费包装类型 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class LineChannelPackagingServiceImpl extends AbstractService<LineChannelPackagingMapper, LineChannelPackagingDO> implements LineChannelPackagingService {

    @Resource
    private LineChannelPackagingMapper lineChannelPackagingMapper;

    @Resource
    private LineChannelPriceStepClearanceMapper lineChannelPriceStepClearanceMapper;

    @Override
    public Long createLineChannelPackaging(LineChannelPackagingCreateReqVO createReqVO) {
        // 插入
        LineChannelPackagingDO lineChannelPackaging = LineChannelPackagingConvert.INSTANCE.convert(createReqVO);
        lineChannelPackagingMapper.insert(lineChannelPackaging);
        // 返回
        return lineChannelPackaging.getId();
    }

    @Override
    public void deleteLineChannelPackaging(Long id) {
        // 校验存在
        this.validateLineChannelPackagingExists(id);
        // 删除
        lineChannelPackagingMapper.deleteById(id);
    }

    private void validateLineChannelPackagingExists(Long id) {
        if (lineChannelPackagingMapper.selectById(id) == null) {
            throw exception(LINE_CHANNEL_PACKAGING_NOT_EXISTS);
        }
    }

    @Override
    public LineChannelPackagingDO getLineChannelPackaging(Long id) {
        return lineChannelPackagingMapper.selectById(id);
    }

    @Override
    public List<LineChannelPackagingDO> getLineChannelPackagingList(Long lineId) {

        List<LineChannelPackagingDO> lineChannelPackagingDOList = lineChannelPackagingMapper.selectList(new LambdaQueryWrapperX<LineChannelPackagingDO>().eq(LineChannelPackagingDO::getLineId, lineId));

        if (CollectionUtil.isNotEmpty(lineChannelPackagingDOList)) {
            for (LineChannelPackagingDO lineChannelPackagingDO : lineChannelPackagingDOList) {
                List<LineChannelPriceStepClearanceDO> lineChannelPriceStepClearanceDOList = lineChannelPriceStepClearanceMapper.selectList(new LambdaQueryWrapperX<LineChannelPriceStepClearanceDO>().eq(LineChannelPriceStepClearanceDO::getPackagingId, lineChannelPackagingDO.getId()));
                lineChannelPackagingDO.setLineChannelPriceStepClearanceDOList(lineChannelPriceStepClearanceDOList);
            }
        }

        return lineChannelPackagingDOList;
    }

    @Override
    public List<LineChannelPackagingDO> getLineChannelPackagingList(Collection<Long> ids) {
        return lineChannelPackagingMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<LineChannelPackagingDO> getLineChannelPackagingPage(LineChannelPackagingQueryVO query, PageVO page) {
        return lineChannelPackagingMapper.selectPage(page, query);
    }

    @Override
    public List<LineChannelPackagingDO> getLineChannelPackagingList(LineChannelPackagingQueryVO query) {
        return lineChannelPackagingMapper.selectList(query);
    }

    @Override
    public List<LineChannelPackagingDO> queryAirClearancePrice(
            List<String> packages,
            Long channelId,
            Long lineId) {
        List<LineChannelPackagingDO> list = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(packages)) {
            for (String packageType : packages) {
                List<LineChannelPackagingDO> packagingDOList =
                        lineChannelPackagingMapper.queryAirClearancePrice(packageType,
                                channelId, lineId);
                if (CollectionUtil.isNotEmpty(packagingDOList)){
                    list.addAll(packagingDOList);
                }
            }
        } else {
            list = lineChannelPackagingMapper.queryAirClearancePrice(null,
                    channelId, lineId);
        }
        return list;
    }
}
