package cn.iocoder.yudao.module.depository.service.warehouse;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridExportReqVO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridPageReqVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfGridDO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridUpdateReqVO;

/**
 * 货架格子 Service 接口
 *
 * @author wlh
 */
public interface ShelfGridService extends IService<ShelfGridDO> {

    /**
     * 生成货架格子
     * @param shelf
     */
    void generateShelfGrid(ShelfDO shelf);

    /**
     * 创建货架格子
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShelfGrid(@Valid ShelfGridCreateReqVO createReqVO);

    /**
     * 更新货架格子
     *
     * @param updateReqVO 更新信息
     */
    void updateShelfGrid(@Valid ShelfGridUpdateReqVO updateReqVO);

    /**
     * 删除货架格子
     *
     * @param id 编号
     */
    void deleteShelfGrid(Long id);

    /**
     * 获得货架格子
     *
     * @param id 编号
     * @return 货架格子
     */
    ShelfGridDO getShelfGrid(Long id);

    /**
     * 获得货架格子列表
     *
     * @param ids 编号
     * @return 货架格子列表
     */
    List<ShelfGridDO> getShelfGridList(Collection<Long> ids);

    /**
     * 获得货架格子分页
     *
     * @param pageReqVO 分页查询
     * @return 货架格子分页
     */
    PageResult<ShelfGridDO> getShelfGridPage(ShelfGridPageReqVO pageReqVO);

    /**
     * 获得货架格子列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 货架格子列表
     */
    List<ShelfGridDO> getShelfGridList(ShelfGridExportReqVO exportReqVO);

}
