package cn.iocoder.yudao.module.depository.service.warehouse;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfExportReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfPageReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfUpdateReqVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;

/**
 * 货架 Service 接口
 *
 * @author wlh
 */
public interface ShelfService extends IService<ShelfDO> {

    /**
     * 创建货架
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShelf(@Valid ShelfCreateReqVO createReqVO);

    /**
     * 更新货架
     *
     * @param updateReqVO 更新信息
     */
    void updateShelf(@Valid ShelfUpdateReqVO updateReqVO);

    /**
     * 删除货架
     *
     * @param id 编号
     */
    void deleteShelf(Long id);

    /**
     * 获得货架
     *
     * @param id 编号
     * @return 货架
     */
    ShelfDO getShelf(Long id);

    /**
     * 获得货架列表
     *
     * @param ids 编号
     * @return 货架列表
     */
    List<ShelfDO> getShelfList(Collection<Long> ids);

    /**
     * 获得货架分页
     *
     * @param pageReqVO 分页查询
     * @return 货架分页
     */
    PageResult<ShelfDO> getShelfPage(ShelfPageReqVO pageReqVO);

    /**
     * 获得货架列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 货架列表
     */
    List<ShelfDO> getShelfList(ShelfExportReqVO exportReqVO);

}
