package cn.iocoder.yudao.module.depository.service.warehouse;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionUpdateReqVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;

/**
 * 仓位管理 Service 接口
 *
 * @author wlh
 */
public interface WarehouseAreaPositionService extends IService<WarehouseAreaPositionDO> {

    /**
     * 创建仓位
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWarehouseAreaPosition(@Valid WarehouseAreaPositionCreateReqVO createReqVO);

    /**
     * 更新仓位
     * @param updateReqVO 更新信息
     */
    void updateWarehouseAreaPosition(@Valid WarehouseAreaPositionUpdateReqVO updateReqVO);

    /**
     * 删除仓位
     * @param id 编号
     */
    void deleteWarehouseAreaPosition(Long id);

    /**
     * 获得仓位
     * @param id 编号
     * @return 仓位
     */
    WarehouseAreaPositionDO getWarehouseAreaPosition(Long id);

    /**
     * 获得仓位列表
     * @param ids 编号
     * @return 仓位列表
     */
    List<WarehouseAreaPositionDO> getWarehouseAreaPositionList(Collection<Long> ids);

    /**
     * 获得仓位分页
     * @param page 分页查询
     * @param query 查询
     * @return 仓位分页
     */
    PageResult<WarehouseAreaPositionDO> getWarehouseAreaPositionPage(WarehouseAreaPositionQueryVO query, PageVO page);

    /**
     * 获得仓位列表, 用于 Excel 导出
     * @param query 查询
     * @return 仓位列表
     */
    List<WarehouseAreaPositionDO> getWarehouseAreaPositionList(WarehouseAreaPositionQueryVO query);

    List<WarehouseAreaPositionDO> getByParentIds(Collection<Long> ids);

    List<WarehouseAreaPositionDO> getByDomainIdList(List<Long> idList);
}
