package cn.iocoder.yudao.module.depository.service.warehouse;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseArea.WarehouseAreaUpdateReqVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaDO;

/**
 * 库区库域 Service 接口
 *
 * @author wlh
 */
public interface WarehouseAreaService extends IService<WarehouseAreaDO> {

    /**
     * 创建库区库域
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWarehouseArea(@Valid WarehouseAreaCreateReqVO createReqVO);

    /**
     * 更新库区库域
     * @param updateReqVO 更新信息
     */
    void updateWarehouseArea(@Valid WarehouseAreaUpdateReqVO updateReqVO);

    /**
     * 删除库区库域
     * @param id 编号
     */
    void deleteWarehouseArea(Long id);

    /**
     * 获得库区库域
     * @param id 编号
     * @return 库区库域
     */
    WarehouseAreaDO getWarehouseArea(Long id);

    /**
     * 获得库区库域列表
     * @param ids 编号
     * @return 库区库域列表
     */
    List<WarehouseAreaDO> getWarehouseAreaList(Collection<Long> ids);

    List<WarehouseAreaDO> getWarehouseAreaByParentIds(Collection<Long> ids);

    /**
     * 获得库区库域分页
     * @param page 分页查询
     * @param query 查询
     * @return 库区库域分页
     */
    PageResult<WarehouseAreaDO> getWarehouseAreaPage(WarehouseAreaQueryVO query, PageVO page);

    /**
     * 获得库区库域列表, 用于 Excel 导出
     * @param query 查询
     * @return 库区库域列表
     */
    List<WarehouseAreaDO> getWarehouseAreaList(WarehouseAreaQueryVO query);

    List<WarehouseAreaBackVO> getWarehouseAreaListV1(WarehouseAreaQueryVO query);

    PageResult<WarehouseAreaBackVO> getWarehouseAreaPageV1(WarehouseAreaQueryVO query, PageVO page);
}
