package cn.iocoder.yudao.module.depository.service.warehouse.impl;

import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridExportReqVO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridPageReqVO;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.ShelfGridUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfGridConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfGridDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.ShelfGridMapper;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfGridService;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.math.BigDecimal;
import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;



/**
 * 货架格子 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class ShelfGridServiceImpl extends AbstractService<ShelfGridMapper, ShelfGridDO> implements ShelfGridService {

    @Resource
    private ShelfGridMapper shelfGridMapper;

    /**
     * 生成货架格子
     * @param shelf  货架
     */
    public void generateShelfGrid(ShelfDO shelf) {
        //eg A区 第6货架 第5层 第1格：A-06-05-01
        String regionCode = shelf.getRegionCode();

        //条码起始号
        Integer barcodeStart = shelf.getBarcodeStart();
        //货架起始号
        Integer shelfStart = shelf.getShelfStart();
        //层数起始号
        Integer layerStart = shelf.getLayerStart();

        //货架数量
        Integer shelfNum = shelf.getShelfNum();
        //货架层数
        Integer shelfLayer = shelf.getShelfLayer();
        //货架格子数
        Integer shelfGridNum = shelf.getShelfGridNum();

        Integer sort = shelf.getSort();

        List<ShelfGridDO> dataList = new ArrayList<>();

        //格子区域编码
        String gridRegionCode;
        //当前格子的货架位置
        int currentShelfNum;
        //当前格子的货架层数位置
        int currentShelfLayerNum;
        for (int i = 0; i < shelfNum; i++) {
            for (int j = 0; j < shelfLayer; j++) {
                for (int k = 0; k < shelfGridNum; k++) {
                    ShelfGridDO gridDO = new ShelfGridDO();
                    gridDO.setShelfId(shelf.getId());

                    currentShelfNum = shelfStart + i;
                    currentShelfLayerNum = layerStart + j;
                    //区域编码  区域编号+货架个数+货架层数+格子数
                    gridRegionCode = regionCode + "-" + currentShelfNum + "-" + currentShelfLayerNum + "-" + (k + 1);
                    gridDO.setRegionCode(gridRegionCode);
                    //货架编号，从条码编号开始
                    gridDO.setShelfNo(barcodeStart);
                    barcodeStart++;
                    gridDO.setVolume(BigDecimal.ZERO);
                    gridDO.setShelfType(shelf.getShelfType());
                    gridDO.setSort(sort);
                    sort++;
                    gridDO.setStatus(shelf.getStatus());

                    dataList.add(gridDO);
                }
            }
        }

        shelfGridMapper.insertBatch(dataList);
    }

    @Override
    public Long createShelfGrid(ShelfGridCreateReqVO createReqVO) {
        // 插入
        ShelfGridDO shelfGrid = ShelfGridConvert.INSTANCE.convert(createReqVO);
        shelfGridMapper.insert(shelfGrid);
        // 返回
        return shelfGrid.getId();
    }

    @Override
    public void updateShelfGrid(ShelfGridUpdateReqVO updateReqVO) {
        // 校验存在
//        this.validateShelfGridExists(updateReqVO.getId());
        // 更新
        ShelfGridDO updateObj = ShelfGridConvert.INSTANCE.convert(updateReqVO);
        shelfGridMapper.updateById(updateObj);
    }

    @Override
    public void deleteShelfGrid(Long id) {
        // 删除
        shelfGridMapper.deleteById(id);
    }



    @Override
    public ShelfGridDO getShelfGrid(Long id) {
        return shelfGridMapper.selectById(id);
    }

    @Override
    public List<ShelfGridDO> getShelfGridList(Collection<Long> ids) {
        return shelfGridMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ShelfGridDO> getShelfGridPage(ShelfGridPageReqVO pageReqVO) {
        return shelfGridMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ShelfGridDO> getShelfGridList(ShelfGridExportReqVO exportReqVO) {
        return shelfGridMapper.selectList(exportReqVO);
    }

}
