package cn.iocoder.yudao.module.depository.service.warehouse.impl;

import cn.iocoder.yudao.module.depository.vo.shelf.ShelfCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfExportReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfPageReqVO;
import cn.iocoder.yudao.module.depository.vo.shelf.ShelfUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.ShelfMapper;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfGridService;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfService;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 货架 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class ShelfServiceImpl extends AbstractService<ShelfMapper, ShelfDO> implements ShelfService {

    @Resource
    private ShelfMapper shelfMapper;
    @Resource
    ShelfGridService shelfGridService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createShelf(ShelfCreateReqVO createReqVO) {
        // 插入
        ShelfDO shelf = ShelfConvert.INSTANCE.convert(createReqVO);
        shelfMapper.insert(shelf);

        //生成货架格子
        shelfGridService.generateShelfGrid(shelf);
        return shelf.getId();
    }

    @Override
    public void updateShelf(ShelfUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateShelfExists(updateReqVO.getId());
        // 更新
        ShelfDO updateObj = ShelfConvert.INSTANCE.convert(updateReqVO);
        shelfMapper.updateById(updateObj);
    }

    @Override
    public void deleteShelf(Long id) {
        // 校验存在
        this.validateShelfExists(id);
        // 删除
        shelfMapper.deleteById(id);
    }

    private void validateShelfExists(Long id) {
        if (shelfMapper.selectById(id) == null) {
            throw exception(SHELF_NOT_EXISTS);
        }
    }

    @Override
    public ShelfDO getShelf(Long id) {
        return shelfMapper.selectById(id);
    }

    @Override
    public List<ShelfDO> getShelfList(Collection<Long> ids) {
        return shelfMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ShelfDO> getShelfPage(ShelfPageReqVO pageReqVO) {
        return shelfMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ShelfDO> getShelfList(ShelfExportReqVO exportReqVO) {
        return shelfMapper.selectList(exportReqVO);
    }

}
