package cn.iocoder.yudao.module.depository.service.warehouse.impl;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseAreaPositionMapper;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 仓位管理 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class WarehouseAreaPositionServiceImpl extends AbstractService<WarehouseAreaPositionMapper, WarehouseAreaPositionDO> implements WarehouseAreaPositionService {

    @Resource
    private WarehouseAreaPositionMapper warehouseAreaPositionMapper;

    @Override
    public Long createWarehouseAreaPosition(WarehouseAreaPositionCreateReqVO createReqVO) {
        // 插入
        WarehouseAreaPositionDO warehouseAreaPosition = WarehouseAreaPositionConvert.INSTANCE.convert(createReqVO);
        warehouseAreaPositionMapper.insert(warehouseAreaPosition);
        // 返回
        return warehouseAreaPosition.getId();
    }

    @Override
    public void updateWarehouseAreaPosition(WarehouseAreaPositionUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateWarehouseAreaPositionExists(updateReqVO.getId());
        // 更新
        WarehouseAreaPositionDO updateObj = WarehouseAreaPositionConvert.INSTANCE.convert(updateReqVO);
        warehouseAreaPositionMapper.updateById(updateObj);
    }

    @Override
    public void deleteWarehouseAreaPosition(Long id) {
        // 校验存在
        this.validateWarehouseAreaPositionExists(id);
        // 删除
        warehouseAreaPositionMapper.deleteById(id);
    }

    private void validateWarehouseAreaPositionExists(Long id) {
        if (warehouseAreaPositionMapper.selectById(id) == null) {
            throw exception(WAREHOUSE_AREA_POSITION_NOT_EXISTS);
        }
    }

    @Override
    public WarehouseAreaPositionDO getWarehouseAreaPosition(Long id) {
        return warehouseAreaPositionMapper.selectById(id);
    }

    @Override
    public List<WarehouseAreaPositionDO> getWarehouseAreaPositionList(Collection<Long> ids) {
        return warehouseAreaPositionMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<WarehouseAreaPositionDO> getWarehouseAreaPositionPage(WarehouseAreaPositionQueryVO query, PageVO page) {
        return warehouseAreaPositionMapper.selectPage(page, query);
    }

    @Override
    public List<WarehouseAreaPositionDO> getWarehouseAreaPositionList(WarehouseAreaPositionQueryVO query) {
        return warehouseAreaPositionMapper.selectList(query);
    }

    @Override
    public List<WarehouseAreaPositionDO> getByParentIds(Collection<Long> ids) {
        return warehouseAreaPositionMapper.selectList(new LambdaQueryWrapperX<WarehouseAreaPositionDO>()
                .in(WarehouseAreaPositionDO::getPid, ids)
        );
    }

    @Override
    public List<WarehouseAreaPositionDO> getByDomainIdList(List<Long> idList) {
        return warehouseAreaPositionMapper.selectList(new LambdaQueryWrapperX<WarehouseAreaPositionDO>()
            .in(WarehouseAreaPositionDO::getDomainId, idList)
        );
    }
}
