package cn.iocoder.yudao.module.depository.service.warehouseLine;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.apollo.core.dto.ChannelDto;
import cn.iocoder.yudao.framework.apollo.core.event.AutoProcessLineChannelOverWeightExceptionEvent;
import cn.iocoder.yudao.framework.apollo.core.event.ChannelDataEvent;
import cn.iocoder.yudao.framework.apollo.core.event.LineChannelUpdateAirWeightEvent;
import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.convert.lineChannelPackaging.LineChannelPackagingConvert;
import cn.iocoder.yudao.module.depository.convert.lineChannelPriceStepClearance.LineChannelPriceStepClearanceConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPackaging.LineChannelPackagingDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPriceStepClearance.LineChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.lineChannelPackaging.LineChannelPackagingMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.lineChannelPriceStepClearance.LineChannelPriceStepClearanceMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseMapper;
import cn.iocoder.yudao.module.depository.dto.LineChannelPackagingWeightDto;
import cn.iocoder.yudao.module.depository.vo.lineChannelPackaging.LineChannelPackagingCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.lineChannelPriceStepClearance.LineChannelPriceStepClearanceCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseLine.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.depository.enums.ErrorCodeConstants.*;

/**
 * 仓库线路关联 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class WarehouseLineServiceImpl extends AbstractService<WarehouseLineMapper, WarehouseLineDO> implements WarehouseLineService {

    @Resource
    private WarehouseLineMapper warehouseLineMapper;

    @Resource
    private LineChannelPackagingMapper lineChannelPackagingMapper;

    @Resource
    private LineChannelPriceStepClearanceMapper lineChannelPriceStepClearanceMapper;

    @Resource
    private WarehouseMapper warehouseMapper;

    @Resource
    private ApplicationContext applicationContext;

    @Override
    public Long createWarehouseLineChannelPackaging(WarehouseLineChannelPackagingCreateReqVO createReqVO) {
        Long lineId = createReqVO.getLineId();

        List<LineChannelPackagingCreateReqVO> packagingCreateReqVOList = createReqVO.getPackagingCreateReqVOList();

        // 校验重复
        checkChannelPackagingTypeRepeat(packagingCreateReqVOList);

        // 查询老数据
        List<LineChannelPackagingDO> lineChannelPackagingListOld = lineChannelPackagingMapper.selectList(new LambdaQueryWrapperX<LineChannelPackagingDO>().eq(LineChannelPackagingDO::getLineId, lineId));

        // TODO 纪录修改前的数据

        // 删除
        lineChannelPackagingMapper.delete(new LambdaQueryWrapperX<LineChannelPackagingDO>().eq(LineChannelPackagingDO::getLineId, lineId));
        lineChannelPriceStepClearanceMapper.delete(new LambdaQueryWrapperX<LineChannelPriceStepClearanceDO>().eq(LineChannelPriceStepClearanceDO::getLineId, lineId));

        // 插入新值
        if (CollectionUtil.isNotEmpty(packagingCreateReqVOList)) {
            for (LineChannelPackagingCreateReqVO lineChannelPackagingCreateReqVO : packagingCreateReqVOList) {
                // 插入包装列表
                LineChannelPackagingDO lineChannelPackagingDO = LineChannelPackagingConvert.INSTANCE.convert(lineChannelPackagingCreateReqVO);
                lineChannelPackagingDO.setLineId(lineId);

                lineChannelPackagingMapper.insert(lineChannelPackagingDO);

                Long packagingDOId = lineChannelPackagingDO.getId();

                // 插入包装列表对应的清关费阶梯价
                List<LineChannelPriceStepClearanceCreateReqVO> priceStepClearanceCreateReqVOList = lineChannelPackagingCreateReqVO.getPriceStepClearanceCreateReqVOList();
                if (CollectionUtil.isNotEmpty(priceStepClearanceCreateReqVOList)) {
                    List<LineChannelPriceStepClearanceDO> lineChannelPriceStepClearanceDOList = LineChannelPriceStepClearanceConvert.INSTANCE.convertList(priceStepClearanceCreateReqVOList)
                            .stream().peek(lineChannelPriceStepClearanceDO -> {
                                lineChannelPriceStepClearanceDO.setLineId(lineId);
                                lineChannelPriceStepClearanceDO.setPackagingId(packagingDOId);
                            }).collect(Collectors.toList());

                    lineChannelPriceStepClearanceMapper.insertBatch(lineChannelPriceStepClearanceDOList);
                }

            }

            // 自动处理商品路线超重异常
            if (CollectionUtil.isNotEmpty(lineChannelPackagingListOld)) {

                // 将老的配置转换为 路线-渠道-包装类型 =>
                List<LineChannelPackagingWeightDto> lineChannelPackagingWeightDtoList = new ArrayList<>();
                for (LineChannelPackagingDO lineChannelPackagingDO : lineChannelPackagingListOld) {

                    String channelIds = lineChannelPackagingDO.getChannelIds();
                    String packagingTypes = lineChannelPackagingDO.getPackagingTypes();

                    if (StringUtils.isNotEmpty(channelIds)) {
                        for (String channelId : channelIds.split(",")) {
                            if (StringUtils.isNotEmpty(packagingTypes)) {
                                for (String packagingType : packagingTypes.split(",")) {
                                    if (lineChannelPackagingWeightDtoList.stream().noneMatch(t -> t.getLineId().equals(lineId)
                                            && t.getChannelIdStr().equals(channelId) && t.getPackagingTypeStr().equals(packagingType))) {
                                        lineChannelPackagingWeightDtoList.add(new LineChannelPackagingWeightDto(lineId, channelId, packagingType, lineChannelPackagingDO.getAirWeightLimit()));
                                    }
                                }
                            }
                        }
                    }
                }


                // 再拿新的去比对
                if (CollectionUtil.isNotEmpty(packagingCreateReqVOList)) {
                    for (LineChannelPackagingCreateReqVO lineChannelPackagingCreateReqVO : packagingCreateReqVOList) {
                        String channelIds = lineChannelPackagingCreateReqVO.getChannelIds();
                        String packagingTypes = lineChannelPackagingCreateReqVO.getPackagingTypes();

                        if (StringUtils.isNotEmpty(channelIds)) {
                            for (String channelId : channelIds.split(",")) {
                                if (StringUtils.isNotEmpty(packagingTypes)) {
                                    for (String packagingType : packagingTypes.split(",")) {
                                        LineChannelPackagingWeightDto lineChannelPackagingWeightDto = lineChannelPackagingWeightDtoList.stream().filter(t -> t.getLineId().equals(lineId)
                                                && t.getChannelIdStr().equals(channelId) && t.getPackagingTypeStr().equals(packagingType)).findFirst().orElse(null);

                                        if (lineChannelPackagingWeightDto != null) {
                                            BigDecimal airWeightLimitOld = lineChannelPackagingWeightDto.getAirWeightLimit();
                                            BigDecimal airWeightLimitNew = lineChannelPackagingCreateReqVO.getAirWeightLimit();

                                            //
                                            BigDecimal weightLimit = null;
                                            if (airWeightLimitNew == null && airWeightLimitNew != null) {
                                                // 删除此路线包装类型的所有未处理超重异常
                                                weightLimit = null;
                                            } else if (airWeightLimitNew != null && airWeightLimitNew == null) {
                                                // do nothing
                                            } else if (airWeightLimitNew != null && airWeightLimitOld != null && airWeightLimitOld.compareTo(airWeightLimitNew) < 0) {
                                                // 新的超重限额更大，则需要找到未处理的，比较后看是否需要处理。
                                                weightLimit = airWeightLimitNew;
                                            }

                                            //  发起自动处理通知
                                            AutoProcessLineChannelOverWeightExceptionEvent event = new AutoProcessLineChannelOverWeightExceptionEvent();
                                            event.setLineId(lineId);
                                            event.setChannelId(Long.parseLong(channelId));
                                            event.setPackingType(packagingType);
                                            event.setAirWeightLimit(weightLimit);
                                            applicationContext.publishEvent(event);
                                        }
                                    }
                                }
                            }
                        }

                    }
                }

            }
        }

        return lineId;
    }

    /**
     * 校验渠道包装类型是否重复
     *
     * @param packagingCreateReqVOList 渠道包装类型请汗就
     */
    private void checkChannelPackagingTypeRepeat(List<LineChannelPackagingCreateReqVO> packagingCreateReqVOList) {
        Map<String, List<String>> map = new HashMap<>();
        // 判断渠道包装类型是否重复
        if (CollectionUtil.isNotEmpty(packagingCreateReqVOList)) {
            for (LineChannelPackagingCreateReqVO lineChannelPackaging : packagingCreateReqVOList) {
                String channelIds = lineChannelPackaging.getChannelIds();
                String packagingTypes = lineChannelPackaging.getPackagingTypes();
                if (StringUtils.isNotEmpty(channelIds) && StringUtils.isNotEmpty(packagingTypes)) {
                    for (String channelId : channelIds.split(",")) {
                        for (String packagingType : packagingTypes.split(",")) {
                            if (map.get(channelId) == null) {
                                map.put(channelId, CollectionUtil.newArrayList(packagingType));
                            } else {
                                map.get(channelId).add(packagingType);
                            }
                        }
                    }
                }
            }
            StringJoiner repeatMsg = new StringJoiner("; ");
            map.forEach((channelId, packagingTypeList) -> {
                // 找到重复的包装类型
                List<String> repeatList = packagingTypeList.stream().filter(t -> packagingTypeList.indexOf(t) != packagingTypeList.lastIndexOf(t)).distinct().collect(Collectors.toList());

                // 将repeatList转换为中文并抛出异常
                if (CollectionUtil.isNotEmpty(repeatList)) {

                    QueryChannelInfoEvent channelInfoEvent = new QueryChannelInfoEvent();
                    channelInfoEvent.setChannelId(Long.parseLong(channelId));
                    applicationContext.publishEvent(channelInfoEvent);
                    String channelInfoEventName = I18nMessage.getLang() == 0 ? channelInfoEvent.getNameZh() : channelInfoEvent.getNameEn();

                    String repeatPackagingTypeName = repeatList.stream().map(s -> {
                        DictDataRespDTO typeDto = DictFrameworkUtils.getDictDataFromCache("packaging_type", s);
                        return I18nMessage.getLang() == 0 ? typeDto.getLabel() : typeDto.getLabelEn();
                    }).collect(Collectors.joining(","));

                    repeatMsg.add(channelInfoEventName + "-" + repeatPackagingTypeName);
                }
            });

            if (StringUtils.isNotEmpty(repeatMsg.toString())) {
                throw exception(WAREHOUSE_LINE_CHANNEL_PACKAGING_TYPE_REPEAT, repeatMsg.toString());
            }

        }
    }


    @Override
    public void deleteWarehouseLine(Long id) {
        // 校验存在
        this.validateWarehouseLineExists(id);
        // 删除
        warehouseLineMapper.deleteById(id);
    }

    private void validateWarehouseLineExists(Long id) {
        if (warehouseLineMapper.selectById(id) == null) {
            throw exception(WAREHOUSE_LINE_NOT_EXISTS);
        }
    }

    @Override
    public WarehouseLineDO getWarehouseLine(Long id) {
        return warehouseLineMapper.selectById(id);
    }

    @Override
    public List<WarehouseLineDO> getWarehouseLineList(Collection<Long> ids) {
        return warehouseLineMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<WarehouseLineBackVO> getWarehouseLinePage(WarehouseLineQueryVO query, PageVO page) {
        IPage<WarehouseLineBackVO> mpPage = MyBatisUtils.buildPage(page);

        mpPage = warehouseLineMapper.getWarehouseLinePage(mpPage, query);

        // 填充渠道名称和id
        List<WarehouseLineBackVO> records = mpPage.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {

            // 找到所有渠道
            ChannelDataEvent event = new ChannelDataEvent();
            applicationContext.publishEvent(event);
            List<ChannelDto> channelDtoList = event.getChannelDtoList();

            for (WarehouseLineBackVO record : records) {
                List<LineChannelPackagingDO> lineChannelPackagingDOList = lineChannelPackagingMapper.selectList(new LambdaQueryWrapperX<LineChannelPackagingDO>().eq(LineChannelPackagingDO::getLineId, record.getId()));
                if (CollectionUtil.isNotEmpty(lineChannelPackagingDOList)) {
                    List<String> channelIdList = new ArrayList<>();
                    for (LineChannelPackagingDO lineChannelPackagingDO : lineChannelPackagingDOList) {
                        String channelIds = lineChannelPackagingDO.getChannelIds();
                        if (StringUtils.isNotEmpty(channelIds)) {
                            Arrays.stream(channelIds.split(",")).filter(t -> !channelIdList.contains(t)).forEach(s -> {
                                channelIdList.add(s);
                            });
                        }
                    }
                    if (CollectionUtil.isNotEmpty(channelIdList)) {
                        // 设置渠道id列表
                        record.setChannelIds(channelIdList.stream().collect(Collectors.joining(",")));

                        // 设置渠道名称
                        StringJoiner channelNamesZh = new StringJoiner(",");
                        StringJoiner channelNamesEn = new StringJoiner(",");
                        for (String s : channelIdList) {
                            ChannelDto channelDto = channelDtoList.stream().filter(t -> t.getChannelId().toString().equals(s)).findFirst().orElse(null);
                            if (channelDto != null) {
                                channelNamesZh.add(channelDto.getNameZh());
                                channelNamesEn.add(channelDto.getNameEn());
                            }
                        }
                        record.setChannelNamesZh(channelNamesZh.toString());
                        record.setChannelNamesEn(channelNamesEn.toString());
                    }
                }


            }

        }
        return PageResult.of(mpPage);
    }

    @Override
    public List<WarehouseLineDO> getWarehouseLineList(WarehouseLineQueryVO query) {
        return warehouseLineMapper.selectList(query);
    }
}
