package cn.iocoder.yudao.module.depository.controller.admin.warehouse;

import cn.iocoder.yudao.module.depository.vo.shelf.*;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;


//@Api(tags = "管理后台 - 货架")
@RestController
@RequestMapping("/ecw/shelf")
@Validated
@Api(tags = "货架")
public class ShelfController {

    @Resource
    private ShelfService shelfService;

    @PostMapping("/create")
    @ApiOperation("创建货架")
    @PreAuthorize("@ss.hasPermission('ecw:shelf:create')")
    public CommonResult<Long> createShelf(@Valid @RequestBody ShelfCreateReqVO createReqVO) {
        return success(shelfService.createShelf(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新货架")
    @PreAuthorize("@ss.hasPermission('ecw:shelf:update')")
    public CommonResult<Boolean> updateShelf(@Valid @RequestBody ShelfUpdateReqVO updateReqVO) {
        shelfService.updateShelf(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除货架")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf:delete')")
    public CommonResult<Boolean> deleteShelf(@RequestParam("id") Long id) {
        shelfService.deleteShelf(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得货架")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf:query')")
    public CommonResult<ShelfRespVO> getShelf(@RequestParam("id") Long id) {
        ShelfDO shelf = shelfService.getShelf(id);
        return success(ShelfConvert.INSTANCE.convert(shelf));
    }

    @GetMapping("/list")
    @ApiOperation("获得货架列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf:query')")
    public CommonResult<List<ShelfRespVO>> getShelfList(@RequestParam("ids") Collection<Long> ids) {
        List<ShelfDO> list = shelfService.getShelfList(ids);
        return success(ShelfConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得货架分页")
    @PreAuthorize("@ss.hasPermission('ecw:shelf:query')")
    public CommonResult<PageResult<ShelfRespVO>> getShelfPage(@Valid ShelfPageReqVO pageVO) {
        PageResult<ShelfDO> pageResult = shelfService.getShelfPage(pageVO);
        return success(ShelfConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出货架 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:shelf:export')")
    @OperateLog(type = EXPORT)
    public void exportShelfExcel(@Valid ShelfExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ShelfDO> list = shelfService.getShelfList(exportReqVO);
        // 导出 Excel
        List<ShelfExcelVO> datas = ShelfConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "货架.xls", "数据", ShelfExcelVO.class, datas);
    }

}
