package cn.iocoder.yudao.module.depository.controller.admin.warehouse;

import cn.iocoder.yudao.module.depository.vo.shelfGrid.*;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfGridConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfGridDO;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfGridService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;


//@Api(tags = "管理后台 - 货架格子")
@RestController
@RequestMapping("/ecw/shelf-grid")
@Validated
@Api(tags = "货架格子")
public class ShelfGridController {

    @Resource
    private ShelfGridService shelfGridService;

    @PostMapping("/create")
    @ApiOperation("创建货架格子")
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:create')")
    public CommonResult<Long> createShelfGrid(@Valid @RequestBody ShelfGridCreateReqVO createReqVO) {
        return success(shelfGridService.createShelfGrid(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新货架格子")
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:update')")
    public CommonResult<Boolean> updateShelfGrid(@Valid @RequestBody ShelfGridUpdateReqVO updateReqVO) {
        shelfGridService.updateShelfGrid(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除货架格子")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:delete')")
    public CommonResult<Boolean> deleteShelfGrid(@RequestParam("id") Long id) {
        shelfGridService.deleteShelfGrid(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得货架格子")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:query')")
    public CommonResult<ShelfGridRespVO> getShelfGrid(@RequestParam("id") Long id) {
        ShelfGridDO shelfGrid = shelfGridService.getShelfGrid(id);
        return success(ShelfGridConvert.INSTANCE.convert(shelfGrid));
    }

    @GetMapping("/list")
    @ApiOperation("获得货架格子列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:query')")
    public CommonResult<List<ShelfGridRespVO>> getShelfGridList(@RequestParam("ids") Collection<Long> ids) {
        List<ShelfGridDO> list = shelfGridService.getShelfGridList(ids);
        return success(ShelfGridConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得货架格子分页")
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:query')")
    public CommonResult<PageResult<ShelfGridRespVO>> getShelfGridPage(@Valid ShelfGridPageReqVO pageVO) {
        PageResult<ShelfGridDO> pageResult = shelfGridService.getShelfGridPage(pageVO);
        return success(ShelfGridConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出货架格子 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:shelf-grid:export')")
    @OperateLog(type = EXPORT)
    public void exportShelfGridExcel(@Valid ShelfGridExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ShelfGridDO> list = shelfGridService.getShelfGridList(exportReqVO);
        // 导出 Excel
        List<ShelfGridExcelVO> datas = ShelfGridConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "货架格子.xls", "数据", ShelfGridExcelVO.class, datas);
    }

}
