package cn.iocoder.yudao.module.depository.controller.admin.warehouse;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 仓位管理")
@RequestMapping("/ecw/warehouse-area-position")
public class WarehouseAreaPositionController {

    @Resource
    private WarehouseAreaPositionService warehouseAreaPositionService;

    @PostMapping("/create")
    @ApiOperation("创建仓位")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:create')")
    public CommonResult<Long> createWarehouseAreaPosition(@Valid @RequestBody WarehouseAreaPositionCreateReqVO createReqVO) {
        return success(warehouseAreaPositionService.createWarehouseAreaPosition(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新仓位")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:update')")
    public CommonResult<Boolean> updateWarehouseAreaPosition(@Valid @RequestBody WarehouseAreaPositionUpdateReqVO updateReqVO) {
        warehouseAreaPositionService.updateWarehouseAreaPosition(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除仓位")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:delete')")
    public CommonResult<Boolean> deleteWarehouseAreaPosition(@RequestParam("id") Long id) {
        warehouseAreaPositionService.deleteWarehouseAreaPosition(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得仓位")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:query')")
    public CommonResult<WarehouseAreaPositionBackVO> getWarehouseAreaPosition(@RequestParam("id") Long id) {
        WarehouseAreaPositionDO warehouseAreaPosition = warehouseAreaPositionService.getWarehouseAreaPosition(id);
        return success(WarehouseAreaPositionConvert.INSTANCE.convert(warehouseAreaPosition));
    }

    @GetMapping("/list")
    @ApiOperation("获得仓位列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:query')")
    public CommonResult<List<WarehouseAreaPositionBackVO>> getWarehouseAreaPositionList(@RequestParam("ids") Collection<Long> ids) {
        List<WarehouseAreaPositionDO> list = warehouseAreaPositionService.getWarehouseAreaPositionList(ids);
        return success(WarehouseAreaPositionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得仓位分页")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:query')")
    public CommonResult<PageResult<WarehouseAreaPositionBackVO>> getWarehouseAreaPositionPage(@Valid WarehouseAreaPositionQueryVO query, PageVO page) {
        PageResult<WarehouseAreaPositionDO> pageResult = warehouseAreaPositionService.getWarehouseAreaPositionPage(query, page);
        PageResult<WarehouseAreaPositionBackVO> backPageResult = WarehouseAreaPositionConvert.INSTANCE.convertPage(pageResult);
        List<WarehouseAreaPositionBackVO> dataList = backPageResult.getList();
        matchChild(dataList);
        return success(backPageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出仓位 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:warehouse-area-position:export')")
    @OperateLog(type = EXPORT)
    public void exportWarehouseAreaPositionExcel(@Valid WarehouseAreaPositionQueryVO query,
              HttpServletResponse response) throws IOException {
        List<WarehouseAreaPositionDO> list = warehouseAreaPositionService.getWarehouseAreaPositionList(query);
        // 导出 Excel
        List<WarehouseAreaPositionBackVO> datas = WarehouseAreaPositionConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "仓位.xls", "数据", WarehouseAreaPositionBackVO.class, datas);
    }

    private void matchChild(List<WarehouseAreaPositionBackVO> dataList) {
        if(CollectionUtil.isNotEmpty(dataList)) {
            List<Long> idList = dataList.stream()
                    .map(WarehouseAreaPositionBackVO::getId)
                    .collect(Collectors.toList());
            List<WarehouseAreaPositionDO> list = warehouseAreaPositionService.getByParentIds(idList);
            Map<Long, List<WarehouseAreaPositionDO>> childGroupMap = list.stream()
                    .collect(Collectors.groupingBy(WarehouseAreaPositionDO::getPid));
            for (WarehouseAreaPositionBackVO backVO : dataList) {
                List<WarehouseAreaPositionDO> childs = childGroupMap.get(backVO.getId());
                List<WarehouseAreaPositionBackVO> childList =  WarehouseAreaPositionConvert.INSTANCE.convertList(childs);
                backVO.setChildren(childList);
            }
        }
    }
}
