package cn.iocoder.yudao.module.depository.controller.admin.warehouseLine;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.convert.lineChannelPackaging.LineChannelPackagingConvert;
import cn.iocoder.yudao.module.depository.convert.lineChannelPriceStepClearance.LineChannelPriceStepClearanceConvert;
import cn.iocoder.yudao.module.depository.convert.warehouseLine.WarehouseLineConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPackaging.LineChannelPackagingDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPriceStepClearance.LineChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.service.lineChannelPackaging.LineChannelPackagingService;
import cn.iocoder.yudao.module.depository.service.warehouseLine.WarehouseLineService;
import cn.iocoder.yudao.module.depository.vo.lineChannelPackaging.LineChannelPackagingBackVO;
import cn.iocoder.yudao.module.depository.vo.lineChannelPackaging.LineChannelPackagingCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.lineChannelPriceStepClearance.LineChannelPriceStepClearanceBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseLine.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 仓库线路")
@RequestMapping("/ecw/warehouse-line")
public class WarehouseLineController {

    @Resource
    private WarehouseLineService warehouseLineService;

    @Resource
    private LineChannelPackagingService lineChannelPackagingService;



    @GetMapping("/page")
    @ApiOperation("获得仓库线路分页")
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:query')")
    public CommonResult<PageResult<WarehouseLineBackVO>> getWarehouseLinePage(@Valid WarehouseLineQueryVO query, PageVO page) {
        PageResult<WarehouseLineBackVO> pageResult = warehouseLineService.getWarehouseLinePage(query, page);
        return success(pageResult);
    }

    @PutMapping("/create-warehouse-line-channel-packaging")
    @ApiOperation("创建仓库线路的渠道包装")
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:create')")
    public CommonResult<Boolean> createWarehouseLineChannelPackaging(@Valid @RequestBody WarehouseLineChannelPackagingCreateReqVO createReqVO) {
        warehouseLineService.createWarehouseLineChannelPackaging(createReqVO);
        return success(true);
    }


    @GetMapping("/get-air-line-channel-packaging-list")
    @ApiOperation("根据线路id渠道列表和清关费（复制也使用）")
    @ApiImplicitParam(name = "lineId", value = "线路id", required = true, example = "2048", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:query')")
    public CommonResult<List<LineChannelPackagingBackVO>> getAirLineChannelPackagingList(@RequestParam("lineId") Long lineId) {
        // List<LineChannelPackagingDO> list = lineChannelPackagingService.getLineChannelPackagingList(lineId);

        List<LineChannelPackagingDO> list = lineChannelPackagingService.queryAirClearancePrice(null, null, lineId);

        List<LineChannelPackagingBackVO> collect = list.stream().map(lineChannelPackagingDO -> {
            LineChannelPackagingBackVO lineChannelPackagingBackVO = LineChannelPackagingConvert.INSTANCE.convert(lineChannelPackagingDO);
            List<LineChannelPriceStepClearanceDO> lineChannelPriceStepClearanceDOList = lineChannelPackagingDO.getLineChannelPriceStepClearanceDOList();
            if (CollectionUtil.isNotEmpty(lineChannelPriceStepClearanceDOList)) {
                List<LineChannelPriceStepClearanceBackVO> lineChannelPriceStepClearanceBackVOS = LineChannelPriceStepClearanceConvert.INSTANCE.convertList2(lineChannelPriceStepClearanceDOList);
                lineChannelPackagingBackVO.setLineChannelPriceStepClearanceBackVOList(lineChannelPriceStepClearanceBackVOS);
            }
            return lineChannelPackagingBackVO;
        }).collect(Collectors.toList());
        return success(collect);
    }


    //
    // @DeleteMapping("/delete")
    // @ApiOperation("删除仓库线路关联")
    // @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:delete')")
    // public CommonResult<Boolean> deleteWarehouseLine(@RequestParam("id") Long id) {
    //     warehouseLineService.deleteWarehouseLine(id);
    //     return success(true);
    // }
    //
    // @GetMapping("/get")
    // @ApiOperation("获得仓库线路关联")
    // @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:query')")
    // public CommonResult<WarehouseLineBackVO> getWarehouseLine(@RequestParam("id") Long id) {
    //     WarehouseLineDO warehouseLine = warehouseLineService.getWarehouseLine(id);
    //     return success(WarehouseLineConvert.INSTANCE.convert(warehouseLine));
    // }


    //
    // @GetMapping("/export-excel")
    // @ApiOperation("导出仓库线路关联 Excel")
    // @PreAuthorize("@ss.hasPermission('depository:warehouse-line:export')")
    // @OperateLog(type = EXPORT)
    // public void exportWarehouseLineExcel(@Valid WarehouseLineQueryVO query,
    //           HttpServletResponse response) throws IOException {
    //     List<WarehouseLineDO> list = warehouseLineService.getWarehouseLineList(query);
    //     // 导出 Excel
    //     List<WarehouseLineBackVO> datas = WarehouseLineConvert.INSTANCE.convertList(list);
    //     ExcelUtils.write(response, "仓库线路关联.xls", "数据", WarehouseLineBackVO.class, datas);
    // }
}
