package cn.iocoder.yudao.module.depository.controller.app.warehouse;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfService;
import cn.iocoder.yudao.module.depository.vo.shelf.*;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Api(tags = "app-web - 货架")
@RestController
@RequestMapping("/ecw/shelf")
@Validated
public class AppShelfController {

    @Resource
    private ShelfService shelfService;


    @GetMapping("/get")
    @ApiOperation("获得货架")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ShelfRespVO> getShelf(@RequestParam("id") Long id) {
        ShelfDO shelf = shelfService.getShelf(id);
        return success(ShelfConvert.INSTANCE.convert(shelf));
    }

    @GetMapping("/list")
    @ApiOperation("获得货架列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthenticated
    public CommonResult<List<ShelfRespVO>> getShelfList(@RequestParam("ids") Collection<Long> ids) {
        List<ShelfDO> list = shelfService.getShelfList(ids);
        return success(ShelfConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得货架分页")
    @PreAuthenticated
    public CommonResult<PageResult<ShelfRespVO>> getShelfPage(@Valid ShelfPageReqVO pageVO) {
        PageResult<ShelfDO> pageResult = shelfService.getShelfPage(pageVO);
        return success(ShelfConvert.INSTANCE.convertPage(pageResult));
    }


}
