package cn.iocoder.yudao.module.depository.controller.app.warehouse;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.depository.service.warehouse.ShelfGridService;
import cn.iocoder.yudao.module.depository.vo.shelfGrid.*;
import cn.iocoder.yudao.module.depository.convert.warehouse.ShelfGridConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.ShelfGridDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;


@Api(tags = "app-web - 货架格子")
@RestController
@RequestMapping("/ecw/shelf-grid")
@Validated
public class AppShelfGridController {

    @Resource
    private ShelfGridService shelfGridService;


    @GetMapping("/get")
    @ApiOperation("获得货架格子")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<ShelfGridRespVO> getShelfGrid(@RequestParam("id") Long id) {
        ShelfGridDO shelfGrid = shelfGridService.getShelfGrid(id);
        return success(ShelfGridConvert.INSTANCE.convert(shelfGrid));
    }

    @GetMapping("/list")
    @ApiOperation("获得货架格子列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthenticated
    public CommonResult<List<ShelfGridRespVO>> getShelfGridList(@RequestParam("ids") Collection<Long> ids) {
        List<ShelfGridDO> list = shelfGridService.getShelfGridList(ids);
        return success(ShelfGridConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得货架格子分页")
    @PreAuthenticated
    public CommonResult<PageResult<ShelfGridRespVO>> getShelfGridPage(@Valid ShelfGridPageReqVO pageVO) {
        PageResult<ShelfGridDO> pageResult = shelfGridService.getShelfGridPage(pageVO);
        return success(ShelfGridConvert.INSTANCE.convertPage(pageResult));
    }


}
