package cn.iocoder.yudao.module.depository.controller.app.warehouse;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseAreaPositionService;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionCreateReqVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionQueryVO;
import cn.iocoder.yudao.module.depository.vo.warehouseAreaPosition.WarehouseAreaPositionUpdateReqVO;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseAreaPositionConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseAreaPositionDO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "app-web - 仓位管理")
@RequestMapping("/ecw/warehouse-area-position")
public class AppWarehouseAreaPositionController {

    @Resource
    private WarehouseAreaPositionService warehouseAreaPositionService;

    @GetMapping("/get")
    @ApiOperation("获得仓位")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<WarehouseAreaPositionBackVO> getWarehouseAreaPosition(@RequestParam("id") Long id) {
        WarehouseAreaPositionDO warehouseAreaPosition = warehouseAreaPositionService.getWarehouseAreaPosition(id);
        return success(WarehouseAreaPositionConvert.INSTANCE.convert(warehouseAreaPosition));
    }

    @GetMapping("/list")
    @ApiOperation("获得仓位列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthenticated
    public CommonResult<List<WarehouseAreaPositionBackVO>> getWarehouseAreaPositionList(@RequestParam("ids") Collection<Long> ids) {
        List<WarehouseAreaPositionDO> list = warehouseAreaPositionService.getWarehouseAreaPositionList(ids);
        return success(WarehouseAreaPositionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得仓位分页")
    @PreAuthenticated
    public CommonResult<PageResult<WarehouseAreaPositionBackVO>> getWarehouseAreaPositionPage(@Valid WarehouseAreaPositionQueryVO query, PageVO page) {
        PageResult<WarehouseAreaPositionDO> pageResult = warehouseAreaPositionService.getWarehouseAreaPositionPage(query, page);
        PageResult<WarehouseAreaPositionBackVO> backPageResult = WarehouseAreaPositionConvert.INSTANCE.convertPage(pageResult);
        List<WarehouseAreaPositionBackVO> dataList = backPageResult.getList();
        matchChild(dataList);
        return success(backPageResult);
    }

    private void matchChild(List<WarehouseAreaPositionBackVO> dataList) {
        if(CollectionUtil.isNotEmpty(dataList)) {
            List<Long> idList = dataList.stream()
                    .map(WarehouseAreaPositionBackVO::getId)
                    .collect(Collectors.toList());
            List<WarehouseAreaPositionDO> list = warehouseAreaPositionService.getByParentIds(idList);
            Map<Long, List<WarehouseAreaPositionDO>> childGroupMap = list.stream()
                    .collect(Collectors.groupingBy(WarehouseAreaPositionDO::getPid));
            for (WarehouseAreaPositionBackVO backVO : dataList) {
                List<WarehouseAreaPositionDO> childs = childGroupMap.get(backVO.getId());
                List<WarehouseAreaPositionBackVO> childList =  WarehouseAreaPositionConvert.INSTANCE.convertList(childs);
                backVO.setChildren(childList);
            }
        }
    }
}
